@@ -1,3 +1,6 @@
+#include <optional>
+#include <vector>
+
 #include <zypp-core/zyppng/core/ByteArray>
 #include <zypp-core/zyppng/rpc/stompframestream.h>
 #include <zypp-core/zyppng/base/private/linuxhelpers_p.h>
@@ -51,6 +54,86 @@ namespace env
   }
 } // namespace env
 
+namespace dbg
+{
+  struct _dumpTransactionStep
+  {
+    _dumpTransactionStep( const zypp::proto::target::TransactionStep *const step_r )
+    : _step { step_r }
+    {}
+    std::ostream & dumpOn( std::ostream & str ) const
+    {
+      str << "TStep(";
+      if ( _step ) {
+        if ( std::holds_alternative<zypp::proto::target::InstallStep>(*_step) ) {
+          const auto &step = std::get<zypp::proto::target::InstallStep>(*_step);
+          str << step.stepId << "|" << step.pathname;
+        } else if ( std::holds_alternative<zypp::proto::target::RemoveStep>(*_step) ) {
+          const auto &step = std::get<zypp::proto::target::RemoveStep>(*_step);
+          str << step.stepId << "|" << step.name << "-" << step.version << "-" << step.release << "." << step.arch;
+        } else {
+          str << "OOPS";
+        }
+      } else {
+        str << "NULL";
+      }
+      return str << ")";
+    }
+    const zypp::proto::target::TransactionStep *const _step;
+  };
+  inline std::ostream & operator<<( std::ostream & str, const _dumpTransactionStep & obj )
+  { return obj.dumpOn( str ); }
+  inline _dumpTransactionStep dump( const zypp::proto::target::TransactionStep *const obj )
+  { return _dumpTransactionStep( obj ); }
+  inline _dumpTransactionStep dump( const zypp::proto::target::TransactionStep & obj )
+  { return _dumpTransactionStep( &obj ); }
+
+  struct _dumpRpmCallbackType
+  {
+    _dumpRpmCallbackType( rpmCallbackType_e flag_r )
+    : _flag { flag_r }
+    {}
+    std::ostream & dumpOn( std::ostream & str ) const
+    {
+      str << "RPMCALLBACK";
+      if ( _flag ) {
+#define OUTS(E) if ( _flag & RPMCALLBACK_##E ) str << "_" << #E
+        OUTS( INST_PROGRESS   );
+        OUTS( INST_START      );
+        OUTS( INST_OPEN_FILE  );
+        OUTS( INST_CLOSE_FILE );
+        OUTS( TRANS_PROGRESS  );
+        OUTS( TRANS_START     );
+        OUTS( TRANS_STOP      );
+        OUTS( UNINST_PROGRESS );
+        OUTS( UNINST_START    );
+        OUTS( UNINST_STOP     );
+        OUTS( UNPACK_ERROR    );
+        OUTS( CPIO_ERROR      );
+        OUTS( SCRIPT_ERROR    );
+        OUTS( SCRIPT_START    );
+        OUTS( SCRIPT_STOP     );
+        OUTS( INST_STOP       );
+        OUTS( ELEM_PROGRESS   );
+#ifdef HAVE_RPM_VERIFY_TRANSACTION_STEP
+        OUTS( VERIFY_PROGRESS );
+        OUTS( VERIFY_START    );
+        OUTS( VERIFY_STOP     );
+#endif
+#undef OUTS
+      } else {
+        str << "_UNKNOWN";
+      }
+      return str;
+    }
+    rpmCallbackType_e _flag;
+  };
+  inline std::ostream & operator<<( std::ostream & str, const _dumpRpmCallbackType & obj )
+  { return obj.dumpOn( str ); }
+  inline _dumpRpmCallbackType dump( rpmCallbackType obj )
+  { return _dumpRpmCallbackType( obj ); }
+}
+
 // this is the order we expect the FDs we need to communicate to be set up
 // by the parent. This is not pretty but it works and is less effort than
 // setting up a Unix Domain Socket and sending FDs over that.
@@ -174,15 +257,38 @@ std::pair<RpmHeader, int> readPackage( rpmts ts_r, const zypp::filesystem::Pathn
   return std::make_pair( h, res );
 }
 
+// Remember 'install source rpm' transaction steps (bsc#1228647, installed separately)
+struct SourcePackageInstallHelper
+{
+  // < transaction step index, path to the .src.rpm >
+  using StepIndex = std::pair<std::size_t,zypp::Pathname>;
+  using StepIndices = std::vector<StepIndex>;
+
+  void remember( std::size_t stepIndex_r, zypp::Pathname pathname_r )
+  {
+    if ( not stepIndices ) stepIndices = StepIndices();
+    stepIndices->push_back( {stepIndex_r, std::move(pathname_r)} );
+  }
+
+  std::optional<StepIndices> stepIndices;   // Remembered source rpm installs
+  std::optional<std::size_t> currentStep;   // Current transaction step index for rpmLibCallback
+};
 
 struct TransactionData {
+  TransactionData( zypp::proto::target::Commit & commitData_r )
+  : commitData { commitData_r }
+  {}
+
   zypp::proto::target::Commit &commitData;
 
   // dbinstance of removals to transaction step index
-  std::unordered_map<int, int> removePckIndex = {};
+  std::unordered_map<int, std::size_t> removePckIndex = {};
 
   // the fd used by rpm to dump script output
   zypp::AutoDispose<FD_t> rpmFd = {};
+
+  // Remember 'install source rpm' transaction steps (bsc#1228647, installed separately)
+  SourcePackageInstallHelper sourcePackageInstallHelper;
 };
 
 int main( int, char ** )
@@ -341,7 +447,7 @@ int main( int, char ** )
   // do we care about knowing the public key?
   const bool allowUntrusted = ( rpmInstFlags & RpmInstFlag::RPMINST_ALLOWUNTRUSTED );
 
-  for ( int i = 0; i < msg.transactionSteps.size(); i++ ) {
+  for ( std::size_t i = 0; i < msg.transactionSteps.size(); ++i ) {
     const auto &step = msg.transactionSteps[i];
 
     if ( std::holds_alternative<zypp::proto::target::InstallStep>(step) ) {
@@ -382,6 +488,14 @@ int main( int, char ** )
         return FailedToReadPackage;
       }
 
+      // bsc#1228647: ::rpmtsInstallElement accepts source rpms without an error, but
+      // is not able to handle them correctly within ::rpmtsRun. We need to remember
+      // them and run ::rpmInstallSourcePackage after the ::rpmtsRun is done.
+      if ( ::headerIsSource( rpmHeader.first.get() ) ) {
+        data.sourcePackageInstallHelper.remember( i, install.pathname );
+        continue;
+      }
+
       const auto res = ::rpmtsAddInstallElement( ts, rpmHeader.first.get(), &step, !install.multiversion, nullptr  );
       if ( res ) {
         ZERR << zypp::str::Format( "Failed to add %1% to the transaction." )% file << std::endl;
@@ -549,6 +663,27 @@ int main( int, char ** )
     return err;
   }
 
+  // bsc#1228647: ::rpmtsInstallElement accepts source rpms without an error, but
+  // is not able to handle them correctly within ::rpmtsRun. We need to remember
+  // them and run ::rpmInstallSourcePackage after the ::rpmtsRun is done.
+  if ( data.sourcePackageInstallHelper.stepIndices ) {
+    ::rpmtsEmpty( ts );   // Re-create an empty transaction set.
+    auto err = NoError;
+    for ( const auto & [stepi,path] : *data.sourcePackageInstallHelper.stepIndices ) {
+      zypp::AutoDispose<FD_t> fd { ::Fopen( path.c_str(), "r" ), ::Fclose };
+      data.sourcePackageInstallHelper.currentStep = stepi; // rpmLibCallback needs to look it up
+      ::rpmRC res = ::rpmInstallSourcePackage( ts, fd, nullptr, nullptr );
+      if ( res != 0 ) {
+        ZERR << "RPM InstallSourcePackage " << path << " failed: " << res << std::endl;
+        err = RpmFinishedWithError;
+      }
+    }
+    data.sourcePackageInstallHelper.currentStep = std::nullopt; // done
+    if ( err != NoError ) {
+      return err;
+    }
+  }
+
   //ZDBG << "Success !!!!" << std::endl;
 
   return NoError;
@@ -596,15 +731,30 @@ void *rpmLibCallback( const void *h, const rpmCallbackType what, const rpm_loff_
   const BinHeader header( (Header)h );
 
   auto iStep = key ? reinterpret_cast< const zypp::proto::target::TransactionStep * >( key ) : nullptr;
-  if ( !iStep && h ) {
-    auto key = headerGetInstance( header.get() );
-    if ( key > 0 ) {
-      auto i = that->removePckIndex.find(key);
-      if ( i != that->removePckIndex.end() )
-        iStep = &that->commitData.transactionSteps[i->second];
+  if ( !iStep ) {
+    if ( that->sourcePackageInstallHelper.currentStep ) {
+      // we're in ::rpmInstallSourcePackage
+      iStep = &that->commitData.transactionSteps[*(that->sourcePackageInstallHelper.currentStep)];
+    }
+    else if ( h ) {
+      auto key = headerGetInstance( header.get() );
+      if ( key > 0 ) {
+        auto i = that->removePckIndex.find(key);
+        if ( i != that->removePckIndex.end() )
+          iStep = &that->commitData.transactionSteps[i->second];
+      }
     }
   }
 
+#if 0
+  // dbg log callback sequence (collapsing RPMCALLBACK_INST_PROGRESS)
+  static rpmCallbackType prev = RPMCALLBACK_UNKNOWN;
+  if ( what != RPMCALLBACK_INST_PROGRESS || prev != RPMCALLBACK_INST_PROGRESS ) {
+    ZDBG << "LibCB " << dbg::dump(what) << ": " << dbg::dump(iStep) << " - "  << header.nvra() << std::endl;
+    prev = what;
+  }
+#endif
+
   const auto &sendEndOfScriptTag = [&](){
     //ZDBG << "Send end of script" << std::endl;
     ::Fflush( that->rpmFd );
@@ -687,7 +837,7 @@ void *rpmLibCallback( const void *h, const rpmCallbackType what, const rpm_loff_
     case RPMCALLBACK_INST_STOP: {
 
       if ( !iStep ) {
-        ZERR << "Could not find package " << header << " in transaction elements for " << what << std::endl;
+        ZERR << "Could not find package " << header << " in transaction elements for " << dbg::dump(what) << std::endl;
         return rc;
       }
 
@@ -729,7 +879,7 @@ void *rpmLibCallback( const void *h, const rpmCallbackType what, const rpm_loff_
     case RPMCALLBACK_UNPACK_ERROR: {
 
       if ( !iStep ) {
-        ZERR << "Could not find package " << header << " in transaction elements for " << what << std::endl;
+        ZERR << "Could not find package " << header << " in transaction elements for " << dbg::dump(what) << std::endl;
         return rc;
       }
 
