@@ -79,7 +79,7 @@ namespace zyppng
      */
     ProvideMediaSpec &addCustomHeaderValue (  const std::string &key, const HeaderValueMap::Value &val );
 
-    zypp::TriBool isSameMedium ( const ProvideMediaSpec &other );
+    zypp::TriBool isSameMedium ( const ProvideMediaSpec &other ) const;
 
   public:
     class Impl;                 ///< Implementation class.
@@ -168,6 +168,12 @@ namespace zyppng
     /** Set the \ref deltafile. */
     ProvideFileSpec &setDeltafile( const zypp::Pathname &path );
 
+    /** The requested file is allowed to be fetched via mirrors ( defaults to true ) */
+    bool mirrorsAllowed() const;
+
+    /** Enables or disables the use of mirrors when fetching this file */
+    ProvideFileSpec &setMirrorsAllowed( bool set = true );
+
     /*!
      * Returns a map of custom key->value pairs that can control special aspects
      * of how the provide operation is processed.
