@@ -149,11 +149,11 @@ namespace ansi
   // enabled via ctor Color::Constant -> Color
   /** \relates ColorTraits<Tp_> SFINAE: hide template signatures unless enum is enabled in \ref ColorTraits */
   template <typename CCC_>
-  using EnableIfCustomColorCtor = typename std::enable_if< ansi::ColorTraits<typename std::decay<CCC_>::type>::customColorCtor >::type;
+  using EnableIfCustomColorCtor = std::enable_if_t< ansi::ColorTraits<std::decay_t<CCC_>>::customColorCtor >;
 
   /** \relates ColorTraits<Tp_> SFINAE: hide template signatures unless enum is enabled in \ref ColorTraits */
   template <typename CCC_>
-  using DisableIfCustomColorCtor = typename std::enable_if< !ansi::ColorTraits<typename std::decay<CCC_>::type>::customColorCtor >::type;
+  using DisableIfCustomColorCtor = std::enable_if_t< !ansi::ColorTraits<std::decay_t<CCC_>>::customColorCtor >;
 
   ///////////////////////////////////////////////////////////////////
   /// \class Color
@@ -185,7 +185,7 @@ namespace ansi
     { Unchanged, Default, Black, Red, Green, Yellow, Blue, Magenta, Cyan, White };
 
     /** Color unique id type */
-    typedef std::uint32_t UidType;
+    using UidType = std::uint32_t;
 
     /** Predefined (foregreound) color contants
      * Intentionally not an <tt>enum class</tt>, so it can be used as
@@ -470,7 +470,7 @@ namespace ansi
   { enum { customColorCtor = true }; };	// enabled via ctor Color::Constant -> Color
 
   // Implememtation after ColorTraits<Color::Constant> instantiation !
-  Color & Color::operator<=( Color::Constant rhs )	{ return *this <= Color( rhs, Bg::Unchanged ); }
+  Color & Color::operator<=( Color::Constant rhs )	{ return this->operator<=( Color( rhs, Bg::Unchanged ) ); }
   Color Color::operator<( Color::Constant rhs ) const	{ return Color(*this) <= rhs; }
 
   /** \relates Color Print the colors SGRsequence if \ref do_colors is \c true */
@@ -566,9 +566,6 @@ namespace ansi
     /** Append a string */
     ColorString & operator<<( const std::string & str_r )
     { _str += str_r; return *this; }
-    /** \overload moving */
-    ColorString & operator<<( std::string && str_r )
-    { _str += std::move(str_r); return *this; }
 
   public:
     /** Assign \ref Color */
