@@ -28,17 +28,17 @@ namespace zypp
   //
   /** Pattern interface.
   */
-  class Pattern : public ResObject
+  class ZYPP_API Pattern : public ResObject
   {
     public:
-      typedef Pattern                  Self;
-      typedef ResTraits<Self>          TraitsType;
-      typedef TraitsType::PtrType      Ptr;
-      typedef TraitsType::constPtrType constPtr;
+      using Self = Pattern;
+      using TraitsType = ResTraits<Self>;
+      using Ptr = TraitsType::PtrType;
+      using constPtr = TraitsType::constPtrType;
 
     public:
-      typedef sat::ArrayAttr<IdString,IdString> NameList;
-      typedef sat::SolvableSet                  Contents;
+      using NameList = sat::ArrayAttr<IdString, IdString>;
+      using Contents = sat::SolvableSet;
 
     public:
       /** */
@@ -118,7 +118,7 @@ namespace zypp
       /** Ctor */
       Pattern( const sat::Solvable & solvable_r );
       /** Dtor */
-      virtual ~Pattern();
+      ~Pattern() override;
   };
   ///////////////////////////////////////////////////////////////////
 
