@@ -28,6 +28,7 @@
 #include <sys/utsname.h>
 #include <functional>
 #include <array>
+#include <climits>
 
 #undef ZYPP_BASE_LOGGER_LOGGROUP
 #define ZYPP_BASE_LOGGER_LOGGROUP "PurgeKernels"
@@ -68,7 +69,10 @@ namespace zypp {
 
         _detectedRunning = true;
 
-        MIL << "Detected running kernel: " << _runningKernelEdition << " " << _runningKernelFlavour << " " << _kernelArch << std::endl;
+        MIL << "Detected running kernel: Flavour: " << _runningKernelFlavour << " Arch: " << _kernelArch << "\n";
+        for ( const auto &edVar : _runningKernelEditionVariants )
+          MIL << "Edition variant: " << edVar << "\n";
+        MIL << std::endl;
 
       } else {
         MIL << "Failed to detect running kernel: " << errno << std::endl;
@@ -88,17 +92,27 @@ namespace zypp {
 
       version = str::regex_substitute( version, str::regex( "-[^-]*$", str::regex::match_extended | str::regex::newline ), "", true );
 
-      // from purge-kernels script, was copied from kernel-source/rpm/mkspec
-      version = str::regex_substitute( version, str::regex( "\\.0-rc", str::regex::match_extended ), ".rc", true );
-      version = str::regex_substitute( version, str::regex( "-rc\\d+", str::regex::match_extended ), "", true );
-      version = str::regex_substitute( version, str::regex( "-", str::regex::match_extended ), ".", true );
 
-      _runningKernelEdition = Edition( version, release );
+      auto makeRcVariant = [ &release ]( std::string myVersion, const std::string &replace ){
+        // from purge-kernels script, was copied from kernel-source/rpm/mkspec
+        myVersion = str::regex_substitute( myVersion, str::regex( "\\.0-rc", str::regex::match_extended ), replace, true );
+        myVersion = str::regex_substitute( myVersion, str::regex( "-rc\\d+", str::regex::match_extended ), "", true );
+        myVersion = str::regex_substitute( myVersion, str::regex( "-", str::regex::match_extended ), ".", true );
+        return Edition( myVersion, release );
+      };
+
+      _runningKernelEditionVariants.clear();
+      _runningKernelEditionVariants.insert( makeRcVariant( version, "~rc") );
+      _runningKernelEditionVariants.insert( makeRcVariant( version, ".rc") );
+
       _runningKernelFlavour = flavour;
 
       MIL << "Parsed info from uname: " << std::endl;
       MIL << "Kernel Flavour: " << _runningKernelFlavour << std::endl;
-      MIL << "Kernel Edition: " << _runningKernelEdition << std::endl;
+      MIL << "Kernel Edition Variants: \n";
+      for ( const auto &var : _runningKernelEditionVariants )
+        MIL << "    " << var << "\n";
+      MIL << std::endl;
     }
 
     bool removePackageAndCheck( const sat::Solvable slv, const std::set<sat::Solvable> &keepList , const std::set<sat::Solvable> &removeList ) const;
@@ -110,7 +124,12 @@ namespace zypp {
     std::set<size_t>  _keepOldestOffsets;
     std::set<Edition> _keepSpecificEditions;
     std::string       _uname_r;
-    Edition           _runningKernelEdition;
+    /*!
+     *  A kernel uname can expand into multiple variants especially with rc kernels,
+     *  older rc releases replaced the upstream tag -rc with .rc, however newer code
+     *  replaces it with ~rc. We need to check against all variants to match the correct one.
+     */
+    std::set<Edition> _runningKernelEditionVariants;
     Flavour           _runningKernelFlavour;
     Arch              _kernelArch;
     std::string       _keepSpec = ZConfig::instance().multiversionKernels();
@@ -139,7 +158,7 @@ namespace zypp {
     MIL << "Request to remove package: " << pi << std::endl;
 
     //list of packages that are allowed to be removed automatically.
-    const str::regex validRemovals("(kernel-syms(-.*)?|kgraft-patch(-.*)?|kernel-livepatch(-.*)?|.*-kmp(-.*)?)");
+    const str::regex validRemovals("(kernel-syms(-.*)?|kgraft-patch(-.*)?|kernel-(.*)-livepatch(-.*)?|kernel-livepatch(-.*)?|.*-kmp(-.*)?)");
 
     if ( pi.status().isLocked() ) {
       MIL << "Package " << pi << " is locked by the user, not removing." << std::endl;
@@ -188,11 +207,35 @@ namespace zypp {
         return false;
       }
 
-      str::smatch what;
-      if ( !str::regex_match( p.name(), what, validRemovals) ) {
-        MIL << "Package " << p << " should not be removed, skipping " << pi << std::endl;
-        pi.statusReset();
-        return false;
+      /*
+       * bsc#1185325 We can not solely rely on name matching to figure out
+       * which packages are kmod's, in SLES from Leap 15.3 forward we have the
+       * kernel-flavour-extra packages ( and others similarly named ) that are basically
+       * a collection of kmod's. So checking the name for .*-kmp(-.*)? is not enough.
+       * We first check if the package provides kmod(*) or ksym(*) and only fall back to name
+       * checking if that is not the case.
+       * Just to be safe I'll leave the regex in the fallback case as well, but it should be completely
+       * redundant now.
+       */
+      bool mostLikelyKmod = false;
+      StrMatcher matchMod( "kmod(*)", Match::GLOB );
+      StrMatcher matchSym( "ksym(*)", Match::GLOB );
+      for ( const auto &prov : p.dep_provides() ) {
+        if ( matchMod.doMatch( prov.detail().name().c_str()) || matchSym.doMatch( prov.detail().name().c_str() ) ) {
+          mostLikelyKmod = true;
+          break;
+        }
+      }
+
+      if ( mostLikelyKmod  ) {
+        MIL << "Package " << p << " is most likely a kmod " << std::endl;
+      } else {
+        str::smatch what;
+        if ( !str::regex_match( p.name(), what, validRemovals) ) {
+          MIL << "Package " << p << " should not be removed, skipping " << pi << std::endl;
+          pi.statusReset();
+          return false;
+        }
       }
     }
 
@@ -209,7 +252,7 @@ namespace zypp {
       for ( const char * suffix : { "-debugsource", "-debuginfo" } ) {
         PoolQuery q;
         q.addKind( zypp::ResKind::package );
-        q.addDependency( sat::SolvAttr::provides, Capability( solvable.name()+suffix, Rel::EQ, solvable.edition() ) );
+        q.addDependency( sat::SolvAttr::dep_provides, Capability( solvable.name()+suffix, Rel::EQ, solvable.edition() ) );
         q.setInstalledOnly();
         q.setMatchExact();
 
@@ -282,7 +325,18 @@ namespace zypp {
       } else {
         str::smatch what;
         if ( !str::regex_match( word, what, specRegex ) ) {
-          _keepSpecificEditions.insert( Edition(word) );
+          // Allow uname_r version numbers by cutting off any postfix that comes after a second dash, including the second dash.
+          // Github-Issue openSUSE/zypper#418
+          std::string_view edition(word);
+          const auto firstDash = word.find_first_of ('-');
+          if ( firstDash != std::string::npos ) {
+            const auto secondDash = word.find_first_of ('-', firstDash+1 );
+            if ( secondDash != std::string::npos ) {
+              WAR << "Ignoring possible flavor postfix:'"<< word.substr (secondDash) <<"' in keep spec: " << word << std::endl;
+              edition = std::string_view( word.c_str (), secondDash );
+            }
+          }
+          _keepSpecificEditions.insert( Edition(IdString(edition)) );
           continue;
         }
 
@@ -332,9 +386,10 @@ namespace zypp {
       removeList.erase( pck );
     };
 
-    const auto versionPredicate = []( const auto &edition ){
-      return [ &edition ]( const auto &elem ) {
-        return versionMatch( edition, elem.first );
+    const auto versionPredicate = []( const auto &editionVariants ){
+      return [ &editionVariants ]( const auto &elem ) {
+        const auto &f = std::bind( versionMatch, _1, elem.first );
+        return std::any_of( editionVariants.begin(), editionVariants.end(), f );
       };
     };
 
@@ -356,14 +411,21 @@ namespace zypp {
              && ( ( archMap.first == _kernelArch && groupInfo.second.groupFlavour == _runningKernelFlavour )
                   || groupInfo.second.groupType == GroupInfo::Sources ) ) {
 
-          MIL << "Matching packages against running kernel "<< _runningKernelEdition << "-" << _runningKernelFlavour << "-" <<_kernelArch << std::endl;
+          MIL << "Matching packages against running kernel "<< _runningKernelFlavour << "-" <<_kernelArch << "\nVariants:\n";
+          for ( const auto &var : _runningKernelEditionVariants )
+            MIL << var << "\n";
+          MIL << std::endl;
 
-          auto it = std::find_if( map.begin(), map.end(), versionPredicate( _runningKernelEdition ) );
+          const auto &editionPredicate = versionPredicate( _runningKernelEditionVariants );
+          auto it = std::find_if( map.begin(), map.end(), editionPredicate );
           if ( it == map.end() ) {
 
             // If we look at Sources we cannot match the flavour but we still want to keep on checking the rest of the keep spec
             if ( groupInfo.second.groupType != GroupInfo::Sources  ) {
-              MIL << "Running kernel "<< _runningKernelEdition << "-" << _runningKernelFlavour << "-" <<_kernelArch << " not installed."<<std::endl;
+              MIL << "Running kernel " << _runningKernelFlavour << "-" <<_kernelArch << "\n";
+              for ( const auto &var : _runningKernelEditionVariants )
+                MIL << " Possible Variant:" << var << "\n";
+              MIL << "Not installed! \n";
               MIL << "NOT removing any packages for flavor "<<_runningKernelFlavour<<"-"<<_kernelArch<<" ."<<std::endl;
 
               for ( const auto &kernelMap : map ) {
@@ -377,7 +439,7 @@ namespace zypp {
             // there could be multiple matches here because of rebuild counter, lets try to find the last one
             MIL << "Found possible running candidate edition: " << it->first << std::endl;
             auto nit = it;
-            for ( nit++ ; nit != map.end() && versionMatch( _runningKernelEdition, nit->first ) ; nit++ ) {
+            for ( nit++ ; nit != map.end() && editionPredicate( *nit ) ; nit++ ) {
               MIL << "Found possible more recent running candidate edition: " << nit->first << std::endl;
               it = nit;
             }
@@ -399,13 +461,14 @@ namespace zypp {
           currOff++;
           currROff--;
 
-          // a kernel package might be explicitely locked by version
+          // a kernel package might be explicitly locked by version
           // We need to go over all package name provides ( provides is named like the package ) and match
           // them against the specified version to know which ones to keep. (bsc#1176740  bsc#1176192)
           std::for_each( kernelMap.second.begin(), kernelMap.second.end(), [ & ]( sat::Solvable solv ){
-            for ( Capability prov : solv.provides() ) {
+            for ( Capability prov : solv.dep_provides() ) {
               if ( prov.detail().name() == solv.name() && _keepSpecificEditions.count( prov.detail().ed() ) ) {
                 markAsKeep( solv );
+                break;
               }
             }
           });
@@ -476,7 +539,7 @@ namespace zypp {
       // not represent the actual rpm package version anymore. ( bsc#1176740 )
       auto currCount = INT_MAX;
       Edition edToUse;
-      for ( Capability prov : installedKrnlPck.provides() ) {
+      for ( Capability prov : installedKrnlPck.dep_provides() ) {
         if ( prov.detail().name() == installedKrnlPck.name() ) {
           if ( edToUse == Edition::noedition ) {
             edToUse = installedKrnlPck.edition();
@@ -498,7 +561,7 @@ namespace zypp {
 
       editionToSolvableMap[edToUse].push_back( installedKrnlPck );
 
-      //in the first step we collect all packages in this list, then later we will remove the packages we want to explicitely keep
+      //in the first step we collect all packages in this list, then later we will remove the packages we want to explicitly keep
       packagesToRemove.insert( installedKrnlPck );
     };
 
@@ -508,7 +571,7 @@ namespace zypp {
     //collect the list of installed kernel packages
     PoolQuery q;
     q.addKind( zypp::ResKind::package );
-    q.addAttribute( sat::SolvAttr::provides, "multiversion(kernel)" );
+    q.addAttribute( sat::SolvAttr::dep_provides, "multiversion(kernel)" );
     q.setInstalledOnly();
     q.setMatchExact();
 
@@ -518,7 +581,7 @@ namespace zypp {
 
       MIL << "Found installed multiversion kernel package " << installedKrnlPck << std::endl;
 
-      if ( installedKrnlPck.provides().matches(Capability("kernel-uname-r")) ) {
+      if ( installedKrnlPck.dep_provides().matches(Capability("kernel-uname-r")) ) {
         MIL << "Identified as a kernel package " << std::endl;
 
         // we group kernel packages by flavour
@@ -542,18 +605,18 @@ namespace zypp {
 
       } else {
 
-        // if adapting the groups do not forget to explicitely handle the group when querying the matches
-        const str::regex explicitelyHandled("kernel-syms(-.*)?|kernel(-.*)?-devel");
+        // if adapting the groups do not forget to explicitly handle the group when querying the matches
+        const str::regex explicitlyHandled("kernel-syms(-.*)?|kernel(-.*)?-devel");
 
         MIL << "Not a kernel package, inspecting more closely " << std::endl;
 
         // we directly handle all noarch packages that export multiversion(kernel)
         if ( installedKrnlPck.arch() == Arch_noarch ) {
 
-          MIL << "Handling package explicitely due to architecture (noarch)."<< std::endl;
+          MIL << "Handling package explicitly due to architecture (noarch)."<< std::endl;
           addPackageToMap( GroupInfo::Sources, installedKrnlPck.name(), "", installedKrnlPck );
 
-        } else if ( str::smatch match; str::regex_match( installedKrnlPck.name(), match, explicitelyHandled ) ) {
+        } else if ( str::smatch match; str::regex_match( installedKrnlPck.name(), match, explicitlyHandled ) ) {
 
           // try to get the flavour from the name
           // if we have a kernel-syms getting no flavour means we have the "default" one, otherwise we use the flavour
@@ -570,10 +633,10 @@ namespace zypp {
           else if ( installedKrnlPck.name() == "kernel-syms" )
             flav = "default";
 
-          MIL << "Handling package explicitely due to name match."<< std::endl;
+          MIL << "Handling package explicitly due to name match."<< std::endl;
           addPackageToMap ( GroupInfo::RelatedBinaries, installedKrnlPck.name(), flav, installedKrnlPck );
         } else {
-          MIL << "Package not explicitely handled" << std::endl;
+          MIL << "Package not explicitly handled" << std::endl;
         }
       }
 
