@@ -31,6 +31,7 @@
 #include <zypp/RepoManagerOptions.h>
 #include <utility>
 #include <zypp-core/ui/ProgressData>
+#include <zypp-core/TriBool.h>
 
 ///////////////////////////////////////////////////////////////////
 namespace zypp
@@ -200,6 +201,17 @@ namespace zypp
                                    const Url &url,
                                    RawMetadataRefreshPolicy policy = RefreshIfNeeded);
 
+    /**
+     * Overload for checkIfToRefreshMetadata using a \ref zypp::MirroredOrigin to check if to refresh
+     * metadata.
+     *
+     * \sa RepoInfo::repoOrigins
+     *
+     */
+    RefreshCheckStatus checkIfToRefreshMetadata( const RepoInfo &info,
+                                   const zypp::MirroredOrigin &origin,
+                                   RawMetadataRefreshPolicy policy = RefreshIfNeeded);
+
     /**
      * \short Path where the metadata is downloaded and kept
      *
@@ -369,7 +381,9 @@ namespace zypp
     */
    void addRepository( const RepoInfo &info,
                        const ProgressData::ReceiverFnc & progressrcv = ProgressData::ReceiverFnc() );
-
+   /** \overload Explicitly request probing or no probing. */
+   void addRepository( const RepoInfo &info, const TriBool & forcedProbe,
+                       const ProgressData::ReceiverFnc & progressrcv = ProgressData::ReceiverFnc() );
    /**
     * \short Adds repositores from a repo file to the list of known repositories.
     * \param url Url of the repo file. The URL is subject to repo variable expansion.
