@@ -489,7 +489,7 @@ namespace zypp
         , repo_add_probe          	( false )
         , repo_refresh_delay      	( 10 )
         , repoLabelIsAlias              ( false )
-        , download_use_deltarpm   	( LIBZYPP_CONFIG_USE_DELTARPM_BY_DEFAULT )
+        , download_use_deltarpm   	( APIConfig(LIBZYPP_CONFIG_USE_DELTARPM_BY_DEFAULT) )
         , download_use_deltarpm_always  ( false )
         , download_media_prefer_download( true )
         , download_mediaMountdir	( "/var/adm/mount" )
@@ -687,10 +687,10 @@ namespace zypp
                 {
                   history_log_path = Pathname(value);
                 }
-                else if ( entry == "techpreview.ZYPP_SINGLE_RPMTRANS" )
+                else if ( entry == "ZYPP_SINGLE_RPMTRANS" || entry == "techpreview.ZYPP_SINGLE_RPMTRANS" )
                 {
-                  DBG << "techpreview.ZYPP_SINGLE_RPMTRANS=" << value << endl;
-                  ::setenv( "ZYPP_SINGLE_RPMTRANS", value.c_str(), 1 );
+                  DBG << "ZYPP_SINGLE_RPMTRANS=" << value << endl;
+                  ::setenv( "ZYPP_SINGLE_RPMTRANS", value.c_str(), 0 );
                 }
                 else if ( entry == "techpreview.ZYPP_MEDIANETWORK" )
                 {
