@@ -29,7 +29,7 @@ namespace zypp
   /// Match mode( Match::GLOB | Match::NOCASE );
   /// \endcode
   ///////////////////////////////////////////////////////////////////
-  class Match
+  class ZYPP_API Match
   {
   private:
     static const int _modemask;
@@ -235,16 +235,16 @@ namespace zypp
   { return Match(lhs) -= rhs; }
 
   /** \relates Match::Mode Stream output */
-  std::ostream & operator<<( std::ostream & str, Match::Mode obj );
+  std::ostream & operator<<( std::ostream & str, Match::Mode obj ) ZYPP_API;
 
   /** \relates Match Stream output */
-  std::ostream & operator<<( std::ostream & str, const Match & obj );
+  std::ostream & operator<<( std::ostream & str, const Match & obj ) ZYPP_API;
 
   ///////////////////////////////////////////////////////////////////
   /// \class MatchException
   /// \brief Exceptions thrown from attribute matching.
   ///////////////////////////////////////////////////////////////////
-  struct MatchException : public Exception
+  struct ZYPP_API MatchException : public Exception
   {
     /** Supplied message. */
     explicit MatchException( const std::string & msg_r ) : Exception( msg_r ) {}
@@ -254,7 +254,7 @@ namespace zypp
   /// \class MatchUnknownModeException
   /// \brief Unknown match mode.
   ///////////////////////////////////////////////////////////////////
-  struct MatchUnknownModeException : public MatchException
+  struct ZYPP_API MatchUnknownModeException : public MatchException
   {
     /** Supplied message. */
     explicit MatchUnknownModeException( const std::string & msg_r ) : MatchException( msg_r ) {}
@@ -267,7 +267,7 @@ namespace zypp
   /// \class MatchInvalidRegexException
   /// \brief Invalid regular expression (failed ::regcomp).
   ///////////////////////////////////////////////////////////////////
-  struct MatchInvalidRegexException : public MatchException
+  struct ZYPP_API MatchInvalidRegexException : public MatchException
   {
     /** Supplied message. */
     explicit MatchInvalidRegexException( const std::string & msg_r ) : MatchException( msg_r ) {}
@@ -299,7 +299,7 @@ namespace zypp
     friend std::ostream & operator<<( std::ostream & str, const StrMatcher & obj );
 
   public:
-    typedef MatchException Exception;
+    using Exception = MatchException;
 
   public:
     /** Implementation  */
