@@ -52,17 +52,17 @@ namespace zypp {
 //
 //	DESCRIPTION :
 //
-MediaHandler::MediaHandler ( Url       url_r,
-                             const Pathname & attach_point_r,
-                             Pathname  urlpath_below_attachpoint_r,
-                             const bool       does_download_r )
+  MediaHandler::MediaHandler (MirroredOrigin origin_r,
+                              const Pathname & attach_point_r,
+                              Pathname  urlpath_below_attachpoint_r,
+                              const bool       does_download_r )
     : _mediaSource()
     , _attachPoint( new AttachPoint())
     , _attachPointHint()
     , _relativeRoot(std::move( urlpath_below_attachpoint_r))
     , _does_download( does_download_r )
     , _attach_mtime(0)
-    , _url(std::move( url_r ))
+    , _origin(std::move( origin_r ))
     , _parentId(0)
 {
   Pathname real_attach_point( getRealPath(attach_point_r.asString()));
@@ -81,8 +81,8 @@ MediaHandler::MediaHandler ( Url       url_r,
     // schemes other than "file" and "dir", if it is absolute.
     //
     if ( !adir.isDir()
-         || (_url.getScheme() != "file"
-             && _url.getScheme() != "dir"
+         || ( _origin.scheme() != "file"
+             && _origin.scheme() != "dir"
              && !real_attach_point.absolute()) )
     {
       ERR << "Provided attach point is not a absolute directory: "
