@@ -11,6 +11,7 @@
 #include "zypp/parser/xml/Reader.h"
 
 #include <zypp-core/ManagedFile.h>
+#include <zypp-core/MirroredOrigin.h>
 #include <zypp-core/zyppng/io/Process>
 #include <zypp-core/zyppng/pipelines/MTry>
 #include <zypp-core/zyppng/pipelines/Algorithm>
@@ -84,7 +85,7 @@ namespace zyppng::RepoManagerWorkflow {
       | and_then([this, providerRef]( MediaHandle medium )
       {
         // first try rpmmd
-        return providerRef->provide( medium, _path/"repodata/repomd.xml", ProvideFileSpec().setCheckExistsOnly( !_targetPath.has_value() ) )
+        return providerRef->provide( medium, _path/"repodata/repomd.xml", ProvideFileSpec().setCheckExistsOnly( !_targetPath.has_value() ).setMirrorsAllowed(false) )
           | and_then( maybeCopyResultToDest("repodata/repomd.xml") )
           | and_then( [](){ return expected<zypp::repo::RepoType>::success(zypp::repo::RepoType::RPMMD); } )
           // try susetags if rpmmd fails and remember the error
@@ -102,7 +103,7 @@ namespace zyppng::RepoManagerWorkflow {
               // any other error, we give up
               return makeReadyResult( expected<zypp::repo::RepoType>::error( ZYPP_FWD_CURRENT_EXCPT() ) );
             }
-            return providerRef->provide( medium, _path/"content", ProvideFileSpec().setCheckExistsOnly( !_targetPath.has_value() ) )
+            return providerRef->provide( medium, _path/"content", ProvideFileSpec().setCheckExistsOnly( !_targetPath.has_value() ).setMirrorsAllowed(false) )
                 | and_then( maybeCopyResultToDest("content") )
                 | and_then( []()->expected<zypp::repo::RepoType>{ return expected<zypp::repo::RepoType>::success(zypp::repo::RepoType::YAST2); } );
           })
@@ -185,6 +186,7 @@ namespace zyppng::RepoManagerWorkflow {
         return probeRepoType( ctx, std::forward<decltype(mediaHandle)>(mediaHandle), path );
     });
   }
+
   }
 
   AsyncOpRef<expected<zypp::repo::RepoType> > probeRepoType(ContextRef ctx, AsyncLazyMediaHandle medium, zypp::Pathname path, std::optional<zypp::Pathname> targetPath)
@@ -504,8 +506,8 @@ namespace zyppng::RepoManagerWorkflow {
       auto helper = std::make_shared<ExHelper>( ExHelper{ refCtx->repoInfo() } );
 
       // the actual logic pipeline, attaches the medium and tries to refresh from it
-      auto refreshPipeline = [ refCtx, progressObserver ]( zypp::Url url ){
-        return refCtx->zyppContext()->provider()->prepareMedia( url, zyppng::ProvideMediaSpec() )
+      auto refreshPipeline = [ refCtx, progressObserver ]( zypp::MirroredOrigin origin ){
+        return refCtx->zyppContext()->provider()->prepareMedia( origin, zyppng::ProvideMediaSpec() )
             | and_then( [ refCtx , progressObserver]( auto mediaHandle ) mutable { return refreshMetadata ( std::move(refCtx), std::move(mediaHandle), progressObserver ); } );
       };
 
@@ -528,8 +530,9 @@ namespace zyppng::RepoManagerWorkflow {
         return true;
       };
 
-      // now go over the urls until we find one that works
-      return refCtx->repoInfo().baseUrls()
+
+      // now go over the url groups until we find one that works
+      return refCtx->repoInfo().repoOrigins()
         | firstOf( std::move(refreshPipeline), expected<RefreshContextRef>::error( std::make_exception_ptr(NotFoundException()) ), std::move(predicate) )
         | [helper]( expected<RefreshContextRef> result ) {
           if ( !result ) {
@@ -885,8 +888,9 @@ namespace zyppng::RepoManagerWorkflow {
 
       ZYPP_ENABLE_LOGIC_BASE(Executor, OpType);
 
-      AddRepoLogic( RepoManagerPtrType &&repoMgrRef, RepoInfo &&info, ProgressObserverRef &&myProgress )
+      AddRepoLogic( RepoManagerPtrType &&repoMgrRef, RepoInfo &&info, ProgressObserverRef &&myProgress, const zypp::TriBool & forcedProbe )
         : _repoMgrRef( std::move(repoMgrRef) )
+        , _doProbeUrl( zypp::indeterminate(forcedProbe) ? _repoMgrRef->options().probe : bool(forcedProbe) )
         , _info( std::move(info) )
         , _myProgress ( std::move(myProgress) )
       {}
@@ -905,7 +909,7 @@ namespace zyppng::RepoManagerWorkflow {
             return makeReadyResult( expected<RepoInfo>::error( ZYPP_EXCPT_PTR(zypp::repo::RepoAlreadyExistsException(_info)) ) );
 
           // check the first url for now
-          if ( _repoMgrRef->options().probe )
+          if ( _doProbeUrl )
           {
             DBG << "unknown repository type, probing" << std::endl;
             return assert_urls(_info)
@@ -938,19 +942,20 @@ namespace zyppng::RepoManagerWorkflow {
       }
 
       RepoManagerPtrType _repoMgrRef;
+      bool _doProbeUrl;  ///< RepoManagerOptions::probe opt. overwritten in by ctor arg \a forcedProbe
       RepoInfo _info;
       ProgressObserverRef _myProgress;
     };
   };
 
-  AsyncOpRef<expected<RepoInfo> > addRepository( AsyncRepoManagerRef mgr, RepoInfo info, ProgressObserverRef myProgress )
+  AsyncOpRef<expected<RepoInfo> > addRepository( AsyncRepoManagerRef mgr, RepoInfo info, ProgressObserverRef myProgress, const zypp::TriBool & forcedProbe )
   {
-    return SimpleExecutor<AddRepoLogic, AsyncOp<expected<RepoInfo>>>::run( std::move(mgr), std::move(info), std::move(myProgress) );
+    return SimpleExecutor<AddRepoLogic, AsyncOp<expected<RepoInfo>>>::run( std::move(mgr), std::move(info), std::move(myProgress), forcedProbe );
   }
 
-  expected<RepoInfo> addRepository( SyncRepoManagerRef mgr, const RepoInfo &info, ProgressObserverRef myProgress )
+  expected<RepoInfo> addRepository( SyncRepoManagerRef mgr, const RepoInfo &info, ProgressObserverRef myProgress, const zypp::TriBool & forcedProbe )
   {
-    return SimpleExecutor<AddRepoLogic, SyncOp<expected<RepoInfo>>>::run( std::move(mgr), RepoInfo(info), std::move(myProgress) );
+    return SimpleExecutor<AddRepoLogic, SyncOp<expected<RepoInfo>>>::run( std::move(mgr), RepoInfo(info), std::move(myProgress), forcedProbe );
   }
 
   namespace {
@@ -1072,9 +1077,9 @@ namespace zyppng::RepoManagerWorkflow {
         using ProvideRes         = typename ProvideType::Res;
 
 
-        RefreshGeoIpLogic( ZyppContextRefType &&zyppCtx, RepoInfo::url_set &&urls )
+        RefreshGeoIpLogic( ZyppContextRefType &&zyppCtx, zypp::MirroredOriginSet &&origins )
           : _zyppCtx( std::move(zyppCtx) )
-          , _urls( std::move(urls) )
+          , _origins( std::move(origins) )
         { }
 
         MaybeAsyncRef<expected<void>> execute() {
@@ -1087,11 +1092,16 @@ namespace zyppng::RepoManagerWorkflow {
           }
 
           std::vector<std::string> hosts;
-          for ( const auto &baseUrl : _urls ) {
-            const auto &host = baseUrl.getHost();
-            if ( zypp::any_of( _zyppCtx->config().geoipHostnames(), [&host]( const auto &elem ){ return ( zypp::str::compareCI( host, elem ) == 0 ); } ) ) {
-              hosts.push_back( host );
-              break;
+          for ( const zypp::MirroredOrigin &origin : _origins ){
+            if ( !origin.schemeIsDownloading () )
+              continue;
+
+            for ( const auto &originEndpoint : origin ) {
+              const auto &host = originEndpoint.url().getHost();
+              if ( zypp::any_of( _zyppCtx->config().geoipHostnames(), [&host]( const auto &elem ){ return ( zypp::str::compareCI( host, elem ) == 0 ); } ) ) {
+                hosts.push_back( host );
+                break;
+              }
             }
           }
 
@@ -1198,7 +1208,7 @@ namespace zyppng::RepoManagerWorkflow {
 
           return std::move(hosts)
           | firstOf( std::move(firstOfCb), false, zyppng::detail::ContinueUntilValidPredicate() )
-          | []( bool foundGeoIP ){
+          | []( bool foundGeoIP ) {
 
             if ( foundGeoIP ) {
               MIL << "Successfully queried GeoIP data." << std::endl;
@@ -1213,7 +1223,7 @@ namespace zyppng::RepoManagerWorkflow {
 
     private:
         ZyppContextRefType _zyppCtx;
-        RepoInfo::url_set  _urls;
+        zypp::MirroredOriginSet _origins;
         zypp::Pathname     _geoIPCache;
 
     };
@@ -1221,14 +1231,24 @@ namespace zyppng::RepoManagerWorkflow {
 
   AsyncOpRef<expected<void> > refreshGeoIPData( ContextRef ctx, RepoInfo::url_set urls )
   {
-    return SimpleExecutor<RefreshGeoIpLogic, AsyncOp<expected<void>>>::run( std::move(ctx), std::move(urls) );
+    return SimpleExecutor<RefreshGeoIpLogic, AsyncOp<expected<void>>>::run( std::move(ctx), zypp::MirroredOriginSet(urls) );
   }
 
   expected<void> refreshGeoIPData( SyncContextRef ctx, RepoInfo::url_set urls )
   {
-    return SimpleExecutor<RefreshGeoIpLogic, SyncOp<expected<void>>>::run( std::move(ctx), std::move(urls) );
+    return SimpleExecutor<RefreshGeoIpLogic, SyncOp<expected<void>>>::run( std::move(ctx), zypp::MirroredOriginSet(urls) );
   }
 
 
+  AsyncOpRef<expected<void> > refreshGeoIPData(ContextRef ctx, zypp::MirroredOriginSet origins)
+  {
+    return SimpleExecutor<RefreshGeoIpLogic, AsyncOp<expected<void>>>::run( std::move(ctx), std::move(origins) );
+  }
+
+  expected<void> refreshGeoIPData(SyncContextRef ctx, zypp::MirroredOriginSet origins)
+  {
+    return SimpleExecutor<RefreshGeoIpLogic, SyncOp<expected<void>>>::run( std::move(ctx), std::move(origins) );
+  }
+
 
 }
