@@ -13,6 +13,7 @@
 #define ZYPP_NG_REPOMANAGER_INCLUDED
 
 #include <utility>
+#include <optional>
 
 #include <zypp/RepoManagerFlags.h>
 #include <zypp/RepoManagerOptions.h>
@@ -39,7 +40,6 @@ namespace zyppng {
   using ServiceInfo         = zypp::ServiceInfo;
   using RepoManagerOptions  = zypp::RepoManagerOptions;
   using PluginRepoverification = zypp_private::repo::PluginRepoverification;
-
   ZYPP_FWD_DECL_TYPE_WITH_REFS( Context );
   ZYPP_FWD_DECL_TYPE_WITH_REFS( SyncContext );
   ZYPP_FWD_DECL_TYPE_WITH_REFS( ProgressObserver );
@@ -376,7 +376,11 @@ namespace zyppng {
     expected<RepoInfo> getRepositoryInfo( const std::string & alias );
     expected<RepoInfo> getRepositoryInfo( const zypp::Url & url, const zypp::url::ViewOption &urlview );
 
-    expected<RefreshCheckStatus> checkIfToRefreshMetadata( const RepoInfo & info, const zypp::Url & url, RawMetadataRefreshPolicy policy );
+    expected<RefreshCheckStatus> checkIfToRefreshMetadata( const RepoInfo & info, const zypp::MirroredOrigin &origin, RawMetadataRefreshPolicy policy );
+
+    expected<RefreshCheckStatus> checkIfToRefreshMetadata( const RepoInfo & info, const zypp::Url & url, RawMetadataRefreshPolicy policy ) {
+      return checkIfToRefreshMetadata ( info, zypp::MirroredOrigin(url), policy );
+    }
 
     /**
      * \short Refresh local raw cache
@@ -399,14 +403,14 @@ namespace zyppng {
 
     std::vector<std::pair<RepoInfo, expected<void> > > refreshMetadata(std::vector<RepoInfo> infos, RawMetadataRefreshPolicy policy, ProgressObserverRef myProgress = nullptr  );
 
-    expected<zypp::repo::RepoType> probe( const zypp::Url & url, const zypp::Pathname & path = zypp::Pathname() ) const;
+    expected<zypp::repo::RepoType> probe( const zypp::MirroredOrigin &origin, const zypp::Pathname & path = zypp::Pathname() ) const;
 
     expected<void> buildCache( const RepoInfo & info, CacheBuildPolicy policy, ProgressObserverRef myProgress = nullptr );
 
     /*!
      * Adds the repository in \a info and returns the updated \ref RepoInfo object.
      */
-    expected<RepoInfo> addRepository( const RepoInfo & info, ProgressObserverRef myProgress = nullptr );
+    expected<RepoInfo> addRepository( const RepoInfo & info, const ProgressObserverRef myProgress = nullptr, const zypp::TriBool & forcedProbe = zypp::indeterminate );
 
     expected<void> addRepositories( const zypp::Url & url, ProgressObserverRef myProgress = nullptr );
 
