@@ -7,7 +7,8 @@
 |                                                                      |
 \---------------------------------------------------------------------*/
 #include "mediafacade.h"
-#include "zypp/media/MediaHandlerFactory.h"
+#include <zypp/media/MediaHandlerFactory.h>
+#include <zypp-media/ng/private/attachedmediainfo_p.h>
 #include <zypp/ZYppCallbacks.h>
 #include <zypp-core/TriBool.h>
 #include <zypp-core/base/userrequestexception.h>
@@ -23,18 +24,19 @@ namespace zyppng {
 
   public:
 
-    AttachedSyncMediaInfo( MediaSyncFacadeRef parentRef, zypp::media::MediaAccessId mediaId, zypp::Url baseUrl, ProvideMediaSpec mediaSpec, const zypp::Pathname &locPath );
+    AttachedSyncMediaInfo( MediaSyncFacadeRef parentRef, zypp::media::MediaAccessId mediaId, zypp::MirroredOrigin origin, ProvideMediaSpec mediaSpec, const zypp::Pathname &locPath );
 
     zypp::media::MediaAccessId mediaId () const;
     const ProvideMediaSpec &spec() const;
-    const zypp::Url &url() const;
+    zypp::Url url() const;
+    const zypp::MirroredOrigin &origin() const;
     const std::optional<zypp::Pathname> &rootPath() const;
     MediaSyncFacadeRef parent() const;
 
     /*!
      * Returns true if \a other requests the same medium as this instance
      */
-    bool isSameMedium ( const std::vector<zypp::Url> &urls, const ProvideMediaSpec &spec );
+    bool isSameMedium ( const zypp::MirroredOrigin &origin, const ProvideMediaSpec &spec );
 
     // ReferenceCounted interface
   protected:
@@ -42,7 +44,7 @@ namespace zyppng {
 
   private:
     zypp::media::MediaAccessId _id;
-    zypp::Url _attachedUrl;
+    zypp::MirroredOrigin _origin;
     ProvideMediaSpec _spec;
     MediaSyncFacadeRef _parent;
     std::optional<zypp::Pathname> _localPath;
@@ -50,9 +52,9 @@ namespace zyppng {
 
   IMPL_PTR_TYPE( AttachedSyncMediaInfo );
 
-  AttachedSyncMediaInfo::AttachedSyncMediaInfo(MediaSyncFacadeRef parentRef, zypp::media::MediaAccessId mediaId, zypp::Url baseUrl, ProvideMediaSpec mediaSpec, const zypp::Pathname &locPath)
+  AttachedSyncMediaInfo::AttachedSyncMediaInfo(MediaSyncFacadeRef parentRef, zypp::media::MediaAccessId mediaId, zypp::MirroredOrigin origin, ProvideMediaSpec mediaSpec, const zypp::Pathname &locPath)
     : _id( mediaId )
-    , _attachedUrl(std::move( baseUrl ))
+    , _origin( std::move( origin ) )
     , _spec(std::move( mediaSpec ))
     , _parent(std::move( parentRef ))
     , _localPath( locPath )
@@ -68,9 +70,14 @@ namespace zyppng {
     return _spec;
   }
 
-  const zypp::Url &AttachedSyncMediaInfo::url() const
+  zypp::Url AttachedSyncMediaInfo::url() const
   {
-    return _attachedUrl;
+    return _origin.authority().url();
+  }
+
+  const zypp::MirroredOrigin &AttachedSyncMediaInfo::origin() const
+  {
+    return _origin;
   }
 
   const std::optional<zypp::Pathname> &AttachedSyncMediaInfo::rootPath() const
@@ -83,14 +90,9 @@ namespace zyppng {
     return _parent;
   }
 
-  bool AttachedSyncMediaInfo::isSameMedium(const std::vector<zypp::Url> &urls, const ProvideMediaSpec &spec) {
-
-    const auto check = _spec.isSameMedium(spec);
-    if ( !zypp::indeterminate (check) )
-      return (bool)check;
-
-    // let the URL rule
-    return ( std::find( urls.begin(), urls.end(), _attachedUrl ) != urls.end() );
+  bool AttachedSyncMediaInfo::isSameMedium(const zypp::MirroredOrigin &origin, const ProvideMediaSpec &spec)
+  {
+    return AttachedMediaInfo::isSameMedium ( _origin, _spec, origin, spec );
   }
 
   void AttachedSyncMediaInfo::unref_to( unsigned int count ) const
@@ -123,9 +125,8 @@ namespace zyppng {
   const zypp::Url &SyncMediaHandle::baseUrl() const
   {
     static zypp::Url invalidHandle;
-    if ( !_data )
-      return invalidHandle;
-    return _data->url();
+    if ( !_data ) return invalidHandle;
+    return _data->origin().authority().url();
   }
 
   const std::optional<zypp::Pathname> &SyncMediaHandle::localPath() const
@@ -152,42 +153,59 @@ namespace zyppng {
 
   ZYPP_IMPL_PRIVATE_CONSTR (MediaSyncFacade)  {  }
 
-  std::vector<zypp::Url> MediaSyncFacade::sanitizeUrls(const std::vector<zypp::Url> &urls) const
+  zypp::MirroredOrigin MediaSyncFacade::sanitizeUrls( const zypp::MirroredOrigin &origin ) const
   {
-    std::vector<zypp::Url> usableMirrs;
-    std::optional<zypp::media::MediaHandlerFactory::MediaHandlerType> handlerType;
-
-    for ( auto mirrIt = urls.begin() ; mirrIt != urls.end(); mirrIt++ ) {
-      const auto &s = zypp::media::MediaHandlerFactory::handlerType ( *mirrIt );
-      if ( !s ) {
-        WAR << "URL: " << *mirrIt << " is not supported, ignoring!" << std::endl;
-        continue;
-      }
-      if ( !handlerType ) {
-        handlerType = *s;
-        usableMirrs.push_back ( *mirrIt );
-      } else {
-        if ( handlerType == *s) {
-          usableMirrs.push_back( *mirrIt );
-        } else {
-          WAR << "URL: " << *mirrIt << " has different handler type than the primary URL: "<< usableMirrs.front() <<", ignoring!" << std::endl;
-        }
-      }
-    }
-
-    if ( !handlerType || usableMirrs.empty() ) {
+    const auto &handlerType = zypp::media::MediaHandlerFactory::handlerType ( origin.authority().url() );
+    if ( !handlerType ) {
+      ERR << "Authority URL: " << origin.authority() << " is not supported!" << std::endl;
       return {};
     }
 
-    return usableMirrs;
+    zypp::MirroredOrigin sanitized( origin.authority() );
+    for ( const auto &mirror : origin.mirrors() ) {
+      const auto &s = zypp::media::MediaHandlerFactory::handlerType ( mirror.url() );
+      if ( !s ) {
+        WAR << "URL: " << mirror << " is not supported, ignoring!" << std::endl;
+        continue;
+      }
+      if ( handlerType == *s) {
+        sanitized.addMirror(mirror);
+      } else {
+        WAR << "URL: " << mirror << " has different handler type than the authority URL: "<< origin.authority() <<", ignoring!" << std::endl;
+      }
+    }
+
+    return sanitized;
   }
 
   expected<MediaSyncFacade::MediaHandle> MediaSyncFacade::attachMedia( const zypp::Url &url, const ProvideMediaSpec &request )
   {
+    return attachMedia( zypp::MirroredOrigin{url}, request );
+  }
 
-    bool isVolatile = url.schemeIsVolatile();
+  expected<MediaSyncFacade::MediaHandle> MediaSyncFacade::attachMedia( const zypp::MirroredOrigin &origin, const ProvideMediaSpec &request )
+  {
+    // rewrite and sanitize the urls if required
+    zypp::MirroredOrigin sanitizedOrigin = sanitizeUrls(origin);
 
-    auto effectiveUrl = url;
+    if ( !sanitizedOrigin.isValid() )
+      return expected<MediaSyncFacade::MediaHandle>::error( ZYPP_EXCPT_PTR ( zypp::media::MediaException("No valid mirrors available") ));
+
+    if ( request.medianr() > 1 ) {
+      for ( auto &ep : sanitizedOrigin )
+        ep.url() = zypp::MediaSetAccess::rewriteUrl( ep.url(), request.medianr() );
+    }
+
+    // first try and find a already attached medium
+    auto i = std::find_if( _attachedMedia.begin (), _attachedMedia.end(), [&]( const AttachedSyncMediaInfo_Ptr &medium ) {
+      return medium->isSameMedium( sanitizedOrigin, request );
+    });
+
+    if ( i != _attachedMedia.end() ) {
+      return expected<MediaSyncFacade::MediaHandle>::success( *i );
+    }
+
+    bool isVolatile = sanitizedOrigin.authority().url().schemeIsVolatile();
 
     std::optional<zypp::media::MediaAccessId> attachId;
     zypp::callback::SendReport<zypp::media::MediaChangeReport> report;
@@ -197,11 +215,7 @@ namespace zyppng {
     do {
       try {
         if ( !attachId ) {
-
-          if ( request.medianr() > 1 )
-            effectiveUrl = zypp::MediaSetAccess::rewriteUrl( effectiveUrl, request.medianr() );
-
-          attachId = mgr.open( effectiveUrl );
+          attachId = mgr.open( sanitizedOrigin );
           if ( !request.mediaFile().empty() ) {
             mgr.addVerifier( *attachId, zypp::media::MediaVerifierRef( new zypp::repo::SUSEMediaVerifier( request.mediaFile(), request.medianr() ) ) );
           }
@@ -211,7 +225,7 @@ namespace zyppng {
         mgr.attach( *attachId );
 
         auto locPath = mgr.localPath( *attachId, "/" );
-        auto attachInfo = AttachedSyncMediaInfo_Ptr( new AttachedSyncMediaInfo( shared_this<MediaSyncFacade>(), *attachId, url, request, locPath )  );
+        auto attachInfo = AttachedSyncMediaInfo_Ptr( new AttachedSyncMediaInfo( shared_this<MediaSyncFacade>(), *attachId, std::move(sanitizedOrigin), request, locPath )  );
         _attachedMedia.push_back( attachInfo );
         return expected<MediaSyncFacade::MediaHandle>::success( std::move(attachInfo) );
 
@@ -280,6 +294,8 @@ namespace zyppng {
             }
           }
 
+          zypp::Url effectiveUrl = sanitizedOrigin.authority().url();
+
           user = report->requestMedia (
             effectiveUrl,
             request.medianr(),
@@ -332,6 +348,12 @@ namespace zyppng {
                 attachId.reset();
               }
 
+              // explicitely setting a URL from the callback wipes all mirrors
+              if ( sanitizedOrigin.authority().url() != effectiveUrl ) {
+                sanitizedOrigin.clearMirrors ();
+                sanitizedOrigin.setAuthority ( effectiveUrl );
+              }
+
               // not attaching, media set will do that for us
               // this could generate uncaught exception (#158620)
               break;
@@ -355,60 +377,6 @@ namespace zyppng {
     } while ( true );
   }
 
-  expected<MediaSyncFacade::MediaHandle> MediaSyncFacade::attachMedia( const std::vector<zypp::Url> &urls, const ProvideMediaSpec &request )
-  {
-    // rewrite and sanitize the urls if required
-    std::vector<zypp::Url> useableUrls = sanitizeUrls(urls);
-
-    if ( useableUrls.empty () )
-      return expected<MediaSyncFacade::MediaHandle>::error( ZYPP_EXCPT_PTR ( zypp::media::MediaException("No valid mirrors available") ));
-
-    // first try and find a already attached medium
-    auto i = std::find_if( _attachedMedia.begin (), _attachedMedia.end(), [&]( const AttachedSyncMediaInfo_Ptr &medium ) {
-      return medium->isSameMedium( useableUrls, request );
-    });
-
-    if ( i != _attachedMedia.end() ) {
-      return expected<MediaSyncFacade::MediaHandle>::success( *i );
-    }
-
-
-    std::exception_ptr lastError;
-    std::exception_ptr jammedError;
-
-
-    // from here call attachMedia with just one URL
-    // catch errors, if one of the URLS returns JAMMED, remember it
-    // continue to try other URLs, if they all fail and JAMMED was remembered
-    // return it, otherwise the last error
-
-    for ( const auto &url : useableUrls ) {
-      try {
-        return expected<MediaSyncFacade::MediaHandle>::success( attachMedia( url, request ).unwrap() );
-      } catch ( zypp::media::MediaJammedException &e) {
-        ZYPP_CAUGHT(e);
-        if ( !jammedError ) jammedError = std::current_exception ();
-      }  catch ( const zypp::Exception &e ) {
-        ZYPP_CAUGHT(e);
-        lastError = std::current_exception();
-      } catch (...) {
-        // didn't work -> clean up
-        lastError = std::current_exception();
-      }
-    }
-
-    if ( jammedError ) {
-      // if we encountered a jammed error return it, there might be still a chance
-      // that invoking the provide again after other pipelines have finished might succeed
-      return expected<MediaSyncFacade::MediaHandle>::error( ZYPP_FWD_EXCPT(jammedError) );
-    } else if ( lastError ) {
-      // if we have a error stored, return that one
-      return expected<MediaSyncFacade::MediaHandle>::error( ZYPP_FWD_EXCPT(lastError) );
-    }
-
-    return expected<MediaSyncFacade::MediaHandle>::error( ZYPP_EXCPT_PTR( zypp::media::MediaException("No URL to attach") ) );
-  }
-
   zyppng::MediaSyncFacade::~MediaSyncFacade()
   {
     // this should never happen because every handle has a reference to the media manager, but still add a debug output
@@ -418,17 +386,17 @@ namespace zyppng {
     }
   }
 
-  expected<MediaSyncFacade::LazyMediaHandle> MediaSyncFacade::prepareMedia(const std::vector<zypp::Url> &urls, const ProvideMediaSpec &request)
+  expected<MediaSyncFacade::LazyMediaHandle> MediaSyncFacade::prepareMedia(const zypp::MirroredOrigin &origin, const ProvideMediaSpec &request)
   {
-    const auto &useableUrls = sanitizeUrls(urls);
-    if ( useableUrls.empty () )
+    const auto &sanitizedOrigin = sanitizeUrls(origin);
+    if ( !sanitizedOrigin.isValid() )
       return expected<MediaSyncFacade::LazyMediaHandle>::error( ZYPP_EXCPT_PTR ( zypp::media::MediaException("No valid mirrors available") ));
-    return expected<LazyMediaHandle>::success( shared_this<MediaSyncFacade>(), std::move(useableUrls), request );
+    return expected<LazyMediaHandle>::success( shared_this<MediaSyncFacade>(), std::move(sanitizedOrigin), request );
   }
 
   expected<MediaSyncFacade::LazyMediaHandle> MediaSyncFacade::prepareMedia(const zypp::Url &url, const ProvideMediaSpec &request)
   {
-    return prepareMedia( std::vector<zypp::Url>{url}, request );
+    return prepareMedia( zypp::MirroredOrigin{url}, request );
   }
 
   expected<MediaSyncFacade::MediaHandle> MediaSyncFacade::attachMediaIfNeeded( LazyMediaHandle lazyHandle)
@@ -439,28 +407,28 @@ namespace zyppng {
 
     MIL << "Attaching lazy medium with label: [" << lazyHandle.spec().label() << "]" << std::endl;
 
-    return attachMedia( lazyHandle.urls(), lazyHandle.spec () )
+    return attachMedia( lazyHandle.origin(), lazyHandle.spec () )
         | and_then([lazyHandle]( MediaHandle handle ) {
           lazyHandle._sharedData->_mediaHandle = handle;
           return expected<MediaHandle>::success( std::move(handle) );
         });
   }
 
-  expected<MediaSyncFacade::Res> MediaSyncFacade::provide(const std::vector<zypp::Url> &urls, const ProvideFileSpec &request)
+  expected<MediaSyncFacade::Res> MediaSyncFacade::provide(const zypp::MirroredOrigin &origin, const ProvideFileSpec &request)
   {
     using namespace zyppng::operators;
 
-    if ( !urls.size() )
+    if ( !origin.isValid() )
       return expected<MediaSyncFacade::Res>::error( ZYPP_EXCPT_PTR ( zypp::media::MediaException("Can not provide a file without a URL.") ));
 
     std::optional<expected<MediaSyncFacade::Res>> lastErr;
-    for ( const zypp::Url& file_url : urls ) {
+    for ( const zypp::OriginEndpoint& file_url : origin ) {
 
-      zypp::Url url(file_url);
-      zypp::Pathname fileName(url.getPathName());
-      url.setPathName ("/");
+      zypp::OriginEndpoint ep(file_url);
+      zypp::Pathname fileName(ep.url().getPathName());
+      ep.url().setPathName ("/");
 
-      expected<MediaSyncFacade::Res> res = attachMedia( url, ProvideMediaSpec( "" ) )
+      expected<MediaSyncFacade::Res> res = attachMedia( zypp::MirroredOrigin(ep), ProvideMediaSpec( "" ) )
           | and_then( [&, this]( const MediaSyncFacade::MediaHandle& handle ) {
               return provide( handle, fileName, request.asOnMediaLocation(fileName, 1));
             });
@@ -476,7 +444,7 @@ namespace zyppng {
       return *lastErr;
 
     // we should not get here, but if we do simply use the first entry to make a not found error
-    zypp::Url url( urls.front() );
+    zypp::Url url( origin.authority().url() );
     zypp::Pathname fileName(url.getPathName());
     url.setPathName ("/");
     return expected<MediaSyncFacade::Res>::error( ZYPP_EXCPT_PTR ( zypp::media::MediaFileNotFoundException( url, fileName )));
@@ -485,7 +453,7 @@ namespace zyppng {
 
   expected<MediaSyncFacade::Res> MediaSyncFacade::provide(const zypp::Url &url, const ProvideFileSpec &request)
   {
-    return provide( std::vector<zypp::Url>{url}, request );
+    return provide( zypp::MirroredOrigin{url}, request );
   }
 
   expected<MediaSyncFacade::Res> MediaSyncFacade::provide(const MediaHandle &attachHandle, const zypp::Pathname &fileName, const ProvideFileSpec &request)
