@@ -92,7 +92,7 @@ namespace zypp
      * }
      * \endcode
      **/
-    class Reader : private zypp::base::NonCopyable
+    class ZYPP_API Reader : private zypp::base::NonCopyable
     {
     public:
       /** Ctor. Setup xmlTextReader and advance to the 1st Node. */
@@ -138,10 +138,10 @@ namespace zypp
 
     public:
       /** */
-      typedef function<bool( Reader & )> ProcessNode;
+      using ProcessNode = function<bool (Reader &)>;
 
       /** */
-      bool foreachNode( ProcessNode fnc_r )
+      bool foreachNode( const ProcessNode& fnc_r )
       {
         if ( _node.isAttribute() )
           nextNode();
@@ -154,7 +154,7 @@ namespace zypp
       }
 
       /** */
-      bool foreachNodeAttribute( ProcessNode fnc_r )
+      bool foreachNodeAttribute( const ProcessNode& fnc_r )
       {
         if ( _node.isAttribute() && ! fnc_r( *this ) )
           return false;
@@ -167,7 +167,7 @@ namespace zypp
       }
 
       /** */
-      bool foreachNodeOrAttribute( ProcessNode fnc_r )
+      bool foreachNodeOrAttribute( const ProcessNode& fnc_r )
       {
         for ( ; ! atEnd(); nextNodeOrAttribute() )
           {
