@@ -29,7 +29,7 @@ void downloadMediaInfo( const Pathname &dest_dir,
 
   //hardcode the max filesize to 20MB, to prevent unlimited data downloads but this limit will
   //never be reached in a sane setup
-  fetcher.enqueue( OnMediaLocation("/media.1/media").setOptional(true).setDownloadSize( ByteCount(20, ByteCount::MB ) ) );
+  fetcher.enqueue( OnMediaLocation("/media.1/media").setOptional(true).setDownloadSize( ByteCount(20, ByteCount::MB ) ).setMirrorsAllowed(false) );
   fetcher.start( dest_dir, media, progressrcv );
   // ready, go!
   fetcher.reset();
