@@ -417,12 +417,11 @@ namespace zypp
         }
       }
 
-      // FIXME we only support the first url for now.
-      if ( info.baseUrlsEmpty() )
+      if ( info.repoOriginsEmpty() )
         ZYPP_THROW(Exception("No url in repository."));
 
       MIL << "provide Package " << _package << endl;
-      Url url = * info.baseUrlsBegin();
+      Url url = info.location();
       try {
       do {
         _retry = false;
@@ -607,6 +606,14 @@ namespace zypp
           ProvideFilePolicy policy;
           policy.progressCB( bind( &RpmPackageProvider::progressDeltaDownload, this, _1 ) );
           delta = _access.provideFile( delta_r.repository().info(), delta_r.location(), policy );
+          // bsc#1245672: delta rpms are optional resources.
+          // ProvideFile however always returns a managed file if no Exception occurred.
+          // So the caller has to check whether the optional file actually exists.
+          if ( not PathInfo(delta).isExist() ) {
+            delta.resetDispose();
+            report()->problemDeltaApply( _("download deltarpm: not found") );
+            return ManagedFile();
+          }
         }
       catch ( const Exception & excpt )
         {
