@@ -21,10 +21,11 @@ namespace zypp
     class RepoMirrorList
     {
       public:
-        RepoMirrorList( const Url & url_r, const Pathname & metadatapath_r, bool mirrorListForceMetalink_r );
+
+        RepoMirrorList( const Url & url_r, const Pathname & metadatapath_r );
 
         RepoMirrorList( const Url & url_r )
-        : RepoMirrorList( url_r, Pathname(), false )
+        : RepoMirrorList( url_r, Pathname() )
         {}
 
         const std::vector<Url> & getUrls() const
@@ -33,6 +34,23 @@ namespace zypp
         std::vector<Url> & getUrls()
         { return _urls; }
 
+        static bool urlSupportsMirrorLink( const zypp::Url &url );
+
+        constexpr static const char *cookieFileName()
+        {
+          return "mirrorlist.cookie";
+        }
+
+        constexpr static const char* cacheFileName()
+        {
+          return "mirrorlist";
+        }
+
+    private:
+        static void saveToCookieFile(const Pathname &path_r, const zypp::Url &url_r );
+        static std::string readCookieFile( const Pathname &path_r );
+        static std::string makeCookie( const zypp::Url &url_r );
+
       private:
         std::vector<Url> _urls;
     };
