@@ -13,9 +13,11 @@
 #define ZYPP_SAT_LOOKUPATTR_H
 
 #include <iosfwd>
+#include <utility>
 
 #include <zypp/base/PtrTypes.h>
 #include <zypp-core/base/DefaultIntegral>
+#include <zypp-core/Globals.h>
 
 #include <zypp/sat/detail/PoolMember.h>
 #include <zypp/sat/SolvAttr.h>
@@ -104,13 +106,13 @@ namespace zypp
      *  }
      * \endcode
      */
-    class LookupAttr
+    class ZYPP_API LookupAttr
     {
       public:
-        typedef MatchException Exception;
+        using Exception = MatchException;
 
       public:
-        typedef unsigned size_type;
+        using size_type = unsigned int;
 
         /** Specify the where to look for the attribule. */
         enum Location {
@@ -122,6 +124,11 @@ namespace zypp
         /** Default ctor finds nothing. */
         LookupAttr();
 
+        LookupAttr(const LookupAttr &) = default;
+        LookupAttr(LookupAttr &&) noexcept = default;
+        LookupAttr &operator=(const LookupAttr &) = default;
+        LookupAttr &operator=(LookupAttr &&) noexcept = default;
+
         /** Lookup \ref SolvAttr in \ref Pool (all repositories). */
         explicit LookupAttr( SolvAttr attr_r, Location = SOLV_ATTR );
         /** \overload SolvAttr within sub-structure \a parent_r. */
@@ -229,10 +236,10 @@ namespace zypp
     ///////////////////////////////////////////////////////////////////
 
     /** \relates LookupAttr Stream output. */
-    std::ostream & operator<<( std::ostream & str, const LookupAttr & obj );
+    std::ostream & operator<<( std::ostream & str, const LookupAttr & obj ) ZYPP_API;
 
     /** \relates LookupAttr Verbose stream output including the query result. */
-    std::ostream & dumpOn( std::ostream & str, const LookupAttr & obj );
+    std::ostream & dumpOn( std::ostream & str, const LookupAttr & obj ) ZYPP_API;
 
     ///////////////////////////////////////////////////////////////////
     //
@@ -262,7 +269,7 @@ namespace zypp
         {}
         /** \copydoc LookupAttr::LookupAttr(SolvAttr) */
         explicit LookupRepoAttr( SolvAttr attr_r )
-        : LookupAttr( attr_r, REPO_ATTR )
+        : LookupAttr( std::move(attr_r), REPO_ATTR )
         {}
         /** \copydoc LookupAttr::LookupAttr(SolvAttr,Repository) */
         explicit LookupRepoAttr( SolvAttr attr_r, Repository repo_r );
@@ -298,14 +305,14 @@ namespace zypp
           {}
           /** Initializes */
           DIWrap( RepoIdType repoId_r, SolvableIdType solvId_r, IdType attrId_r,
-                  const std::string & mstring_r = std::string(), int flags_r = 0 );
+                  std::string  mstring_r = std::string(), int flags_r = 0 );
           /** \overload to catch \c NULL \a mstring_r. */
           DIWrap( RepoIdType repoId_r, SolvableIdType solvId_r, IdType attrId_r,
                   const char * mstring_r, int flags_r = 0 );
           DIWrap( const DIWrap & rhs );
           ~DIWrap();
         public:
-          void swap( DIWrap & rhs )
+          void swap( DIWrap & rhs ) noexcept
           {
             if ( &rhs != this ) // prevent self assign!
             {
@@ -476,7 +483,7 @@ namespace zypp
           * If \ref sat::SolvAttr::allAttr is passed, \ref subBegin is returned.
           * \see \ref solvAttrSubEntry
          */
-        iterator subFind( SolvAttr attr_r ) const;
+        iterator subFind( const SolvAttr& attr_r ) const;
         /** \overload Extending the current attribute name with by \c ":attrname_r".
          *
          * This assumes a sub-structur \c "update:reference" has attributes
@@ -573,7 +580,7 @@ namespace zypp
     ///////////////////////////////////////////////////////////////////
 
     /** \relates LookupAttr::iterator Stream output. */
-    std::ostream & operator<<( std::ostream & str, const LookupAttr::iterator & obj );
+    std::ostream & operator<<( std::ostream & str, const LookupAttr::iterator & obj ) ZYPP_API;
 
     ///////////////////////////////////////////////////////////////////
 
