@@ -43,12 +43,12 @@ namespace zypp
      *
      * Explicitly shared singleton \ref Pool::instance.
      */
-    class Pool : protected detail::PoolMember
+    class ZYPP_API Pool : protected detail::PoolMember
     {
       public:
-        typedef detail::SolvableIterator SolvableIterator;
-        typedef zypp::detail::RepositoryIterator     RepositoryIterator;
-        typedef detail::size_type        size_type;
+        using SolvableIterator = detail::SolvableIterator;
+        using RepositoryIterator = zypp::detail::RepositoryIterator;
+        using size_type = detail::size_type;
 
       public:
         /** Singleton ctor. */
@@ -256,7 +256,7 @@ namespace zypp
          * \see \ref ZConfig::multiversionSpec
          */
         //@{
-        typedef SolvableSet MultiversionList;
+        using MultiversionList = SolvableSet;
         const MultiversionList & multiversion() const;
         /** \deprecated Legacy, use multiversion().empty() instead. */
         ZYPP_DEPRECATED bool multiversionEmpty() const { return multiversion().empty(); }
@@ -288,7 +288,7 @@ namespace zypp
     ///////////////////////////////////////////////////////////////////
 
     /** \relates Pool Stream output */
-    std::ostream & operator<<( std::ostream & str, const Pool & obj );
+    std::ostream & operator<<( std::ostream & str, const Pool & obj ) ZYPP_API;
 
     /** \relates Pool */
     inline bool operator==( const Pool & lhs, const Pool & rhs )
