@@ -169,8 +169,10 @@ namespace zypp
             yOut << YAML::Value << YAML::BeginMap;
             yOut << YAML::Key << "alias" << YAML::Value << myRepo.alias();
             yOut << YAML::Key << "url" << YAML::BeginSeq;
-            for ( auto itUrl = myRepoInfo.baseUrlsBegin(); itUrl != myRepoInfo.baseUrlsEnd(); ++itUrl ) {
-              yOut << YAML::Value << itUrl->asString();
+            for ( const auto &origin : myRepoInfo.repoOrigins () ) {
+              for ( const auto &originEndpoint : origin ) {
+                yOut << YAML::Value << originEndpoint.url().asString();
+              }
             }
             yOut << YAML::EndSeq;
             yOut << YAML::Key << "path" << YAML::Value << myRepoInfo.path().asString();
