@@ -13,9 +13,9 @@
 #include <string>
 #include <iostream>
 
-#include "zypp/target/TargetException.h"
+#include <zypp/target/TargetException.h>
+#include <zypp/base/Gettext.h>
 
-using namespace std;
 
 ///////////////////////////////////////////////////////////////////
 namespace zypp
@@ -23,9 +23,14 @@ namespace zypp
   namespace target {
   /////////////////////////////////////////////////////////////////
 
+    TargetAbortedException::TargetAbortedException()
+      : TargetAbortedException ( _("Installation has been aborted as directed.") )
+    { }
+
     std::ostream & TargetAbortedException::dumpOn( std::ostream & str ) const
     {
-      return str << "Installation aborted by user";
+        //call base implementation, do not hardcode a string, do not blame user ( fixes bnc#978193 )
+        return TargetException::dumpOn( str );
     }
 
 
