@@ -90,9 +90,6 @@ namespace target
 {
 namespace rpm
 {
-  const callback::UserData::ContentType InstallResolvableReport::contentRpmout( "rpmout","installpkg" );
-  const callback::UserData::ContentType RemoveResolvableReport::contentRpmout( "rpmout","removepkg" );
-
 namespace
 {
 const char* quoteInFilename_m = "\'\"";
@@ -110,7 +107,7 @@ inline std::string rpmQuoteFilename( const Pathname & path_r )
 }
 
 
-  /** Workaround bnc#827609 - rpm needs a readable pwd so we
+  /** Workaround bsc#827609 - rpm needs a readable pwd so we
    * chdir to /. Turn realtive pathnames into absolute ones
    * by prepending cwd so rpm still finds them
    */
@@ -128,6 +125,30 @@ inline std::string rpmQuoteFilename( const Pathname & path_r )
 #endif
     return path_r;	// no problem with absolute pathnames
   }
+
+  /**
+   * Workaround bsc#1216091: In rpm-4.18.0 `rpm --root /mnt --runposttrans`
+   * does not execute the scripts chrooted. We cant use it.
+   */
+  inline bool workaroundDUMPPOSTTRANS_BUG_1216091( bool checkit_r=false )
+  {
+    auto checkit = []()->bool {
+      bool broken = false;
+      librpmDb::db_const_iterator it( "/" );
+      if ( it.findPackage( "rpm" )
+        && it->tag_edition() == Edition("4.18.0")
+        && not it->tag_provides().count( Capability("rpm_fixed_runposttrans") ) ) {
+        WAR << "Workaround broken rpm --runposttrans" << endl;
+        broken = true;
+      }
+      return broken;
+    };
+
+    static bool broken = false;
+    if ( checkit_r )
+      broken = checkit();
+    return broken;
+  }
 }
 
 struct KeyRingSignalReceiver : callback::ReceiveReport<KeyRingSignals>
@@ -259,6 +280,7 @@ RpmDb::db_const_iterator RpmDb::dbConstIterator() const
 //
 void RpmDb::initDatabase( Pathname root_r, bool doRebuild_r )
 {
+  workaroundDUMPPOSTTRANS_BUG_1216091( /*checkit_r*/true );
   ///////////////////////////////////////////////////////////////////
   // Check arguments
   ///////////////////////////////////////////////////////////////////
@@ -449,7 +471,7 @@ namespace
   void computeKeyRingSync( std::set<Edition> & rpmKeys_r, std::list<PublicKeyData> & zyppKeys_r )
   {
     ///////////////////////////////////////////////////////////////////
-    // Remember latest release and where it ocurred
+    // Remember latest release and where it occurred
     struct Key
     {
       Key()
@@ -1684,11 +1706,7 @@ void RpmDb::doInstallPackage( const Pathname & filename, RpmInstFlags flags, Rpm
 
   // run rpm
   RpmArgVec opts;
-#if defined(WORKAROUNDDUMPPOSTTRANS_BUG_1216091)
-  if ( postTransCollector_r && _root == "/" ) {
-#else
-  if ( postTransCollector_r ) {
-#endif
+  if ( postTransCollector_r && ( _root == "/" || not workaroundDUMPPOSTTRANS_BUG_1216091() ) ) {
     opts.push_back("--define");           // bsc#1041742: Attempt to delay %transfiletrigger(postun|in) execution iff rpm supports it.
     opts.push_back("_dump_posttrans 1");  // Old rpm ignores the --define, new rpm injects 'dump_posttrans:' lines to collect and execute later.
   }
@@ -1899,11 +1917,7 @@ void RpmDb::doRemovePackage( const std::string & name_r, RpmInstFlags flags, Rpm
 
   // run rpm
   RpmArgVec opts;
-#if defined(WORKAROUNDDUMPPOSTTRANS_BUG_1216091)
-  if ( postTransCollector_r && _root == "/" ) {
-#else
-  if ( postTransCollector_r ) {
-#endif
+  if ( postTransCollector_r && ( _root == "/" || not workaroundDUMPPOSTTRANS_BUG_1216091() ) ) {
     opts.push_back("--define");           // bsc#1041742: Attempt to delay %transfiletrigger(postun|in) execution iff rpm supports it.
     opts.push_back("_dump_posttrans 1");  // Old rpm ignores the --define, new rpm injects 'dump_posttrans:' lines to collect and execute later.
   }
@@ -2011,6 +2025,15 @@ int RpmDb::runposttrans( const Pathname & filename_r, const std::function<void(c
   MIL << "RpmDb::runposttrans(" << filename_r << ")" << endl;
 
   RpmArgVec opts;
+#if 1
+  // Bug 1218459 rpm scriptlets left over after snapshot updates
+  // Until 'rpm --runposttrans' is fixed to properly indicate script
+  // execution without -vv we redirect rpm's tmpdir. This will wipe
+  // the rpm-tmp.* files 'rpm -vv' otherwise leaves in /var/tmp.
+  std::string _tmppath { "_tmppath " + filename_r.dirname().asString() };
+  opts.push_back("--define");
+  opts.push_back(_tmppath.c_str());
+#endif
   opts.push_back("-vv");  // want vverbose output to see scriptlet execution in the log
   opts.push_back("--runposttrans");
   opts.push_back(filename_r.c_str());
@@ -2021,6 +2044,7 @@ int RpmDb::runposttrans( const Pathname & filename_r, const std::function<void(c
   // to figure out which script is currently executed. Otherwise we
   // can't tell which output belongs to which script.
   static const str::regex rx( "^D: (%.*): (scriptlet start|running .* scriptlet)" );
+  static const str::regex rx2( "^Running (%[^)]*[)])$" );
   str::smatch what;
   std::string line;
   bool silent = true; // discard everything before 1st scriptlet
@@ -2032,12 +2056,21 @@ int RpmDb::runposttrans( const Pathname & filename_r, const std::function<void(c
     if ( str::startsWith( line, "D:" ) ) {  // rpm debug output
       if ( str::regex_match( line, what, rx ) ) {
         // forward ripoff header
+        DBG << "Verbose RIPOFF:"+what[1] << endl;
         output_r( "RIPOFF:"+what[1] );
         if ( silent )
           silent = false;
       }
       continue;
     }
+    if ( str::regex_match( line, what, rx2 ) ) {  // preliminary for 1218459, but rpm needs fixing
+      // forward ripoff header
+      DBG << "NonVerbose RIPOFF:"+what[1] << endl;
+      // output_r( "RIPOFF:"+what[1] );
+      // if ( silent )
+      //   silent = false;
+      continue;
+    }
     if ( silent ) {
       continue;
     }
