@@ -14,6 +14,7 @@
 
 #include <iosfwd>
 #include <string>
+#include <zypp/Globals.h>
 
 ///////////////////////////////////////////////////////////////////
 namespace zypp
@@ -51,7 +52,7 @@ namespace zypp
     ///////////////////////////////////////////////////////////////////
 
     /** \relates Status Enum value as string. */
-    std::string asString( const Status & obj );
+    std::string asString( const Status & obj ) ZYPP_API;
 
     ///////////////////////////////////////////////////////////////////
 
